package com.cloudera.keytrustee.util;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * Simple generic container to check for updates.
 */
public class ConfigUpdate <ObjectType> {

    private static final Log LOG = LogFactory.getLog(ConfigUpdate.class);
    
    private final String configKey;
    private final ObjectType oldObject;
    private final ObjectType newObject;
    private final String updateMessage;
    private final boolean supportUnset;
    private final boolean sensitiveValue;

  public ConfigUpdate(String configKey, ObjectType existingObject, ObjectType newObject,
                      String updateMessage) {
    this(configKey, existingObject, newObject, updateMessage, false);
  }

  public ConfigUpdate(String configKey, ObjectType existingObject, ObjectType newObject,
                      String updateMessage, boolean supportUnset) {
    this(configKey, existingObject, newObject, updateMessage, supportUnset, false);
    
  }
  
  public ConfigUpdate(String configKey, ObjectType existingObject, ObjectType newObject, 
                                                  String updateMessage, boolean supportUnset, boolean sensitiveValue) {
        this.configKey = configKey;
        this.oldObject = existingObject;
        this.newObject = newObject;
        this.updateMessage = updateMessage;
        this.supportUnset = supportUnset;
        this.sensitiveValue = sensitiveValue;
    }
    
    public String getMessage() {
        if (updateRequired() && !this.sensitiveValue) return this.updateMessage + " (" + this.configKey + 
                                                        ") is being modified from " 
                                                        + oldObject.toString() + " to " + newObject.toString();
        if (updateRequired() && this.sensitiveValue) return this.updateMessage + " (" + this.configKey +
                                                        ") is being modified. From/to values will not be printed as " +
                                                        "this value is sensitive.";
        else return "No modification of configuration for " + this.updateMessage + " required.";
    }
    
    public boolean updateRequired() {
        if (null == oldObject && null != newObject) return true;
        if (null != oldObject && null == newObject) {
          if (!this.supportUnset) {
            if (LOG.isDebugEnabled()) LOG.debug("Config value [" + this.configKey + "], " +
                    "currently set to [" + this.oldObject + "] cannot be unset (not supported).");
            return false;
          }
          return true;
        } 
        if (null == oldObject && null == newObject) return false;
        if (!oldObject.equals(newObject)) return true;
        return false;
    }
    
    public ObjectType getOldObject() {
        return this.oldObject;
    }
    
    public ObjectType getNewObject() {
        return this.newObject;
    }
  
    public String getConfigKey() { return this.configKey; }
  
    public ObjectType getUpdate() { 
      if (LOG.isDebugEnabled()) {
        LOG.debug("getUpdate called on configKey [ " + this.configKey + "]. Old value: [" + this.oldObject + "], New value: [" + this.newObject + "]");
      }
      if (this.updateRequired()) {
        return newObject;
      } else {
        return oldObject;
      }
    }
  
    public boolean isSupportUnset() {
      return this.supportUnset;
    }
  
    public boolean isSensitiveValue() {
      return this.sensitiveValue;
    }
    
}
